"""
WillofSteel API Wrapper
~~~~~~~~~~~~~~~~~~~~~~~

A wrapper for the Will of Steel API

:copyright: (C) 2024-present ItsNeil
:license: MIT, see LICENSE for more details

"""

__title__ = "willofsteel"
__author__ = "ItsNeil"
__license__ = "MIT"
__copyright__ = "Copyright 2024-present ItsNeil"
__version__ = "0.0.1a"

from .wrapper import *


"""
WillofSteel API Wrapper
~~~~~~~~~~~~~~~~~~~~~~~

A wrapper for the Will of Steel API

:copyright: (C) 2024-present ItsNeil
:license: MIT, see LICENSE for more details

"""
from __future__ import annotations
from typing import Literal
import requests
import asyncio
import atexit

from .types import Player, Alliance

BASE = "https://api.wos.itsneil.tech"

class Wrapper:
    def __init__(self, api_key: str):
        self.api_key = api_key
        self.headers = {
            "API-Key": self.api_key,
            "Content-Type": "application/json"
        }
        # self._verify_key() # not implemented in API as of now

    def _verify_key(self) -> None:
        raise NotImplementedError("Key Verification not available.")

    def get_player(self) -> dict:
        response = self.request("GET", "/player", self.headers)    
        # There can not be a 403 error raised as we already verified the key.
        if response.status == 200:
            data = response.json()
            return Player.from_response(data)

    def get_alliance(self) -> dict:
        response = self.request("GET", "/alliance", headers=self.headers)
        status = response.status
        if status == 400:
            # raise NotInAlliance() # custom errors still left to do
            raise KeyError
        data = response.json()
        return Alliance.from_response(data)

    def update_alliance_name(self, new_name: str) -> bool:
        if len(new_name) > 32: # this is not an official limit. just a wrapper limit for now
            return KeyError # return an error which says it cant be more than 32 chars 
        headers = self.headers
        headers["update_type"] = "name"
        headers["new_name"] = new_name
        response = self.request("POST", "/alliance", headers=headers)
        status = response.status
        if status == 200:
            return True
        else:
            return False # TODO: add better error management
            
    def update_alliance_user_limit(self, new_limit: str) -> bool:
        if new_limit > 50:
            return KeyError # return an error which says it cant be more than 50 
        headers = self.headers
        headers["update_type"] = "limit"
        headers["new_limit"] = new_limit
        response = self.request("POST", "/alliance", headers=headers)        
        status = response.status
        if status == 200:
            return True
        else:
            return False # TODO: add better error management
            
    def request(self, method: Literal["GET", "POST"], route: str, headers: dict):
        headers = {
            "User-Agent": "Will of Steel API Wrapper",
            "API-Key": self.api_key,
            "Content-Type": "application/json"
        }
        url = BASE + route

        if method not in ["GET", "POST"]:
            return KeyError("Invalid Method")

        response = requests.request(method, url, headers=headers)
        response.status = response.status_code
        return response



from setuptools import setup
import re

requirements = [
    "aiohttp>=3.7.4,<4"
]

extras_require = {
    'voice': ['PyNaCl>=1.3.0,<1.6'],
    'docs': [
        'sphinx==4.4.0',
        'sphinxcontrib_trio==1.1.2',
        # TODO: bump these when migrating to a newer Sphinx version
        'sphinxcontrib-websupport==1.2.4',
        'sphinxcontrib-applehelp==1.0.4',
        'sphinxcontrib-devhelp==1.0.2',
        'sphinxcontrib-htmlhelp==2.0.1',
        'sphinxcontrib-jsmath==1.0.1',
        'sphinxcontrib-qthelp==1.0.3',
        'sphinxcontrib-serializinghtml==1.1.5',
        'typing-extensions>=4.3,<5',
        'sphinx-inline-tabs==2023.4.21',
    ],
    'speed': [
        'orjson>=3.5.4',
        'aiodns>=1.1',
        'Brotli',
        'cchardet==2.1.7; python_version < "3.10"',
    ],
    'test': [
        'coverage[toml]',
        'pytest',
        'pytest-asyncio',
        'pytest-cov',
        'pytest-mock',
        'typing-extensions>=4.3,<5',
        'tzdata; sys_platform == "win32"',
    ],
}

packages = [
    'willofsteel',
]

readme = ''
with open('README.rst') as f:
    readme = f.read()

setup(
    name='willofsteel',
    author='ItsNeil',
    url='https://github.com/WillofStel-Devs/api-wrapper',
    version="0.0.1",
    packages=packages,
    license='MIT',
    description='A Python wrapper for the Discord API',
    long_description=readme,
    long_description_content_type='text/x-rst',
    include_package_data=True,
    install_requires=requirements,
    extras_require=extras_require,
    python_requires='>=3.8.0',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
        'Typing :: Typed',
    ],
)